/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.kenai.com
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.examples.nodes.example7.view;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import javax.swing.Action;
import org.openide.util.lookup.ServiceProvider;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.netbeans.nodes.role.ActionProvider;
import it.tidalwave.netbeans.capabilitiesprovider.CapabilitiesProvider;
import it.tidalwave.netbeans.capabilitiesprovider.CapabilitiesProviderSupport;
import it.tidalwave.netbeans.examples.nodes.example6.model.FileModel4;
import static it.tidalwave.role.BinaryReadable.BinaryReadable;
import static it.tidalwave.role.TextReadable.TextReadable;
import static it.tidalwave.netbeans.util.EnhancedLookups.*;

/***********************************************************************************************************************
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@ServiceProvider(service=CapabilitiesProvider.class)
public class ActionProviderProvider extends CapabilitiesProviderSupport<FileModel4>  
  {
    private static class FileActionProvider implements ActionProvider
      {
        @Nonnull
        private final Object object;
        
        @Nonnull
        private final List<Action> actions = new ArrayList<Action>();    
        
        private boolean initialized = false;

        public FileActionProvider (final @Nonnull Object object) 
          {
            this.object = object;
          }
        
        @Override @Nonnull
        public Action getPreferredAction() 
          throws NotFoundException
          {
            initialize();
            return NotFoundException.throwWhenEmpty(actions, "no actions").get(0);
          }

        @Override @Nonnull
        public Collection<? extends Action> getActions() 
          {
            initialize();
            return new CopyOnWriteArrayList<Action>(actions);
          }
        
        private void initialize()
          {
            if (!initialized)
              {
                initialized = true;  
                
                if (lookup(object, TextReadable, null) != null)
                  {
                    actions.add(new OpenTextReadableAction(lookup(object, TextReadable)));  
                  }

                else if (lookup(object, BinaryReadable, null) != null)
                  {
                    actions.add(new OpenBinaryReadableAction(lookup(object, BinaryReadable)));  
                  }
              }
          }
      };

    @Override @Nonnull
    public Collection<? extends Object> createCapabilities (final @Nonnull FileModel4 owner)
      {
        return Collections.singletonList(new FileActionProvider(owner));
      }
  }
